classdef FindSources
    
    properties
        configFile = Configuration;
        svAPI = API;
    end
    
    methods
        
        %function that calculate the emmision rate of a single source over
        %the entire grid, based on the ambient data measured by a single
        %sensor.
        %input: location and ambient data of the sensor
        %output: grid with estimated source emission rates, on each grid
        %cell.
        function sourceGrid = getGridEmissionData(obj,x_sensor, y_sensor, measurement_sensor)
            sizeOfStudyArea = obj.configFile.GRID_SIZE;
            sourceGrid = zeros(sizeOfStudyArea,sizeOfStudyArea);
            
            for i=1:sizeOfStudyArea
                for j = 1:sizeOfStudyArea
                    sourceGrid(j,i) = obj.svAPI.calculateSourceCon(x_sensor,y_sensor,i,j,measurement_sensor);
                end
            end
            
            %end of getGridEmissionData
        end
        
        
        %function that calculate  ambient data grid from a single source.
        %input: source location and emission rate.
        %output: grid cell of ambient data on
        function ambientDataGrid  = getAmbientDataGrid(obj, x_source, y_source, emissionRate)
            sizeOfStudyArea = obj.configFile.GRID_SIZE;
            ambientDataGrid = zeros(sizeOfStudyArea,sizeOfStudyArea);
            
            for i=1:sizeOfStudyArea
                for j = 1:sizeOfStudyArea
                    ambientDataGrid(j,i) = obj.svAPI.calculateSensorCon(i, j, x_source, y_source, emissionRate);
                end
            end
            
            %end of getAmbientDataGrid
        end
        
        
        
        
        % calculate the grid cell of two sources based on one sensor
        % measurement
        
        function [gridCellSingleSensorEstimation] = getSingleSensorEstimationGrid(obj, currentSensor)
            
            
            x_coordinates_current_sensor = currentSensor(1,1);
            y_coordinates_current_sensor = currentSensor(1,2);
            ambient_data_current_sensor = currentSensor(1,3);
            
            sizeOfStudyArea = obj.configFile.GRID_SIZE;
            gridCellSingleSensorEstimation = cell(sizeOfStudyArea,sizeOfStudyArea);
            for i=1:sizeOfStudyArea
                
                for j = 1:sizeOfStudyArea
                    x_temp_source1 = i;
                    y_temp_source1 = j;
                    Q_temp_source1 = 5e+5;
                    
                    source1_contribution = obj.svAPI.calculateSensorCon(x_coordinates_current_sensor,y_coordinates_current_sensor,x_temp_source1,y_temp_source1,Q_temp_source1);
                    
                    source2_contribution = ambient_data_current_sensor - source1_contribution;
                    
                    %outer loop: columns,X coordinates
                    Q_temp_source2 = zeros(sizeOfStudyArea,sizeOfStudyArea);
                    for k=1:sizeOfStudyArea
                        %inner loop: rows, Y  coordinates
                        for l=1:sizeOfStudyArea
                            Q_temp_source2(l,k) = obj.svAPI.calculateSourceCon(x_coordinates_current_sensor,y_coordinates_current_sensor,k,l,source2_contribution);
                        end
                    end
                    
                    gridCellSingleSensorEstimation{j,i} = Q_temp_source2;
                    
                    
                end
            end
            
            %end of function getSingleSensorEstimationGrid
        end
        
        function sum_all_measurements = getEstimationOfEntireGrid(obj, mesAllSensors, numOfsensors)
            sizeOfStudyArea = obj.configFile.GRID_SIZE;
            sum_all_measurements = cell(sizeOfStudyArea,sizeOfStudyArea);
            for h = 1:numOfsensors
                current_sensor = mesAllSensors(1,h);
                
                for i=1:sizeOfStudyArea
                    for j=1:sizeOfStudyArea
                        
                        temp =  current_sensor{1,1}{i,j};
                        for k=1:sizeOfStudyArea
                            for l=1:sizeOfStudyArea
                                sum_all_measurements{i,j}{k,l}{1,h} = temp(k,l);
                            end
                        end
                    end
                end
            end
            %end of function
        end
        
        
        %get the STD matrix
        function std_mat = createSTDMatrix(obj, sum_all_measurements)
            sizeOfStudyArea = obj.configFile.GRID_SIZE;
            maximumNumberOfErrorMeasurements =  obj.configFile.SIZE_OF_DEFECTED_MEASURES;
            std_mat = cell(sizeOfStudyArea,sizeOfStudyArea);
            
            for i=1:sizeOfStudyArea
                for j =1:sizeOfStudyArea
                    for k=1:sizeOfStudyArea
                        for l=1:sizeOfStudyArea
                            sum_estimation_all_sensors_one_cell = cell2mat(sum_all_measurements{i,j}{k,l}(1,:));
                            if sum(sum_estimation_all_sensors_one_cell <= 0)>maximumNumberOfErrorMeasurements
                                sum_estimation_all_sensors_one_cell = NaN;
                            else
                                sum_estimation_all_sensors_one_cell(sum_estimation_all_sensors_one_cell== 0) = [];
                            end
                            std_mat{i,j}{k,l} = obj.getSTD(sum_estimation_all_sensors_one_cell);%std(cell2mat(sum_all_measurements{i,j}{k,l}(1,:)));
                        end
                    end
                end
            end
            %end of function
        end
        
        function stdOfCell = getSTD(~,singleCell)
            
            stdOfCell = std(singleCell);
            %end of function
        end
        
        
    end
    
end